@@ -1,6 +1,6 @@
 /* FIGARO'S PASSWORD MANAGER 2 (FPM2)
  * Copyright (C) 2000 John Conneely
- * Copyright (C) 2008,2009 Aleš Koval
+ * Copyright (C) 2008-2010 Aleš Koval
  *
  * FPM is open source / free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,7 +24,6 @@
 #endif
 
 #include <gtk/gtk.h>
-#include <string.h>
 #include <stdlib.h>
 
 #include "fpm.h"
@@ -119,20 +118,6 @@ on_clist_main_button_press_event       (GtkWidget       *widget,
                                         GdkEventButton  *event,
                                         gpointer         user_data)
 {
- // Limit dblclick on mouse button 1 only
-/* if (event->button == 1) {
-  if((g_timer_elapsed(glb_timer_click, NULL) < FPM_MAX_DOUBLE_CLICK_TIME) && (glb_click_row == glb_cur_row))
-  {
-    glb_click_count++;
-  }
-  else
-  {
-    g_timer_reset(glb_timer_click);
-    glb_click_count = 1;
-    glb_click_row = glb_cur_row;
-  }
- } */
-
  if (event->button == 3) {
     glb_click_btn = TRUE;
  }
@@ -160,15 +145,6 @@ on_clist_main_button_release_event     (GtkWidget       *widget,
                                         GdkEventButton  *event,
                                         gpointer         user_data)
 {
- // Limit dblclick on mouse button 1 only
-/* if (event->button == 1) {
-  if((g_timer_elapsed(glb_timer_click, NULL) < FPM_MAX_DOUBLE_CLICK_TIME) &&
-     (glb_click_count > 1))
-  {
-    fpm_double_click(glb_edit_data);
-  }
- } */
-
  if (event->button == 2) {
   if((g_timer_elapsed(glb_timer_click, NULL) < FPM_MAX_DOUBLE_CLICK_TIME) &&
      (glb_click_count > 1))
@@ -194,13 +170,13 @@ on_clist_main_select_row               (GtkTreeView *treeview,
   if (gtk_tree_selection_get_selected(selection, &model, &selectedIter)) {
 
 	gtk_tree_model_get(model, &selectedIter,
-	  FPM_COL_DATA_POINTER, &glb_edit_data,
+	  FPM_DATA_POINTER, &glb_edit_data,
 	  -1);
 
     row = gtk_tree_model_get_string_from_iter(model, &selectedIter);
     glb_cur_row = atoi(row);
 	g_free(row);
-	
+
     if (glb_click_btn) {
 
 		if (gui->context_menu == NULL)
@@ -212,7 +188,7 @@ on_clist_main_select_row               (GtkTreeView *treeview,
 	glb_click_btn = FALSE;
     }
   }
-
+  fpm_sensitive_menu(TRUE);
 }
 
 
@@ -461,6 +437,9 @@ on_button_cpw_cancel_clicked           (GtkWidget       *widget,
     /* User is canceling from changing a password.  Just close the screen. */
     gtk_widget_destroy(glb_win_misc);
   }
+
+  return TRUE;
+
 }
 
 
@@ -516,9 +495,10 @@ on_item_delete_activate                (GtkMenuItem     *menuitem,
 	gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
 	glb_num_row--;
 
-
 	glb_edit_data = NULL;
-	state->dirty = TRUE;
+	fpm_sensitive_menu(FALSE);
+//	state->dirty = TRUE;
+	fpm_dirty(TRUE);
 	if (ini->save_on_delete)
 	    passfile_save(glb_filename);
 
@@ -542,12 +522,9 @@ on_export1_activate                    (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
 	GtkWidget     *dialog;
-	GtkWidget     *hbox;
 	GtkWidget     *combo_box;
 	GtkWidget     *combo_box_category;
-	GtkFileFilter *filter;
 	const gchar *fpm_directory;
-	int            i;
 
 	dialog = gtk_file_chooser_dialog_new (
 			_("Export passwords"),
@@ -593,7 +570,7 @@ on_export1_activate                    (GtkMenuItem     *menuitem,
 	0,0,15,0
 	);
 	combo_box_category = gtk_combo_box_new_text ();
-	fpm_populate_combo_box(combo_box_category, state->category);
+	fpm_populate_combo_box(GTK_COMBO_BOX(combo_box_category), state->category);
 	gtk_table_attach_defaults (GTK_TABLE(table), GTK_WIDGET(combo_box_category), 1,2,2,3);
 
 	gtk_widget_show_all (table);
@@ -624,7 +601,6 @@ on_import_passwords1_activate          (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
     GtkWidget *dialog;
-    GtkFileFilter *filter;
     const gchar *fpm_directory;
     GtkWidget *combo_box;
     GtkWidget *combo_box_category;
@@ -659,7 +635,7 @@ on_import_passwords1_activate          (GtkMenuItem     *menuitem,
     gtk_table_attach (GTK_TABLE(table), gtk_label_new (_("Category:")), 0,1,2,3, 0,0,15,0);
 
     combo_box_category = gtk_combo_box_new_text ();
-    fpm_populate_combo_box(combo_box_category, state->category);
+    fpm_populate_combo_box(GTK_COMBO_BOX(combo_box_category), state->category);
     gtk_combo_box_remove_text (GTK_COMBO_BOX(combo_box_category), 0);
     gtk_combo_box_remove_text (GTK_COMBO_BOX(combo_box_category), 0);
     gtk_combo_box_insert_text (GTK_COMBO_BOX(combo_box_category), 0, _("Do not change the category of items and import as it is."));
@@ -737,7 +713,6 @@ on_open1_activate                      (GtkMenuItem     *menuitem,
 				      GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
 				      NULL);
 
-//    fpm_directory = g_strjoin(NULL, g_get_home_dir(), "/.fpm2/", NULL);
     fpm_directory = g_build_filename (g_get_home_dir(), FPM_DIR, NULL);
 
     gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), fpm_directory);
@@ -766,7 +741,6 @@ on_save_as1_activate                   (GtkMenuItem     *menuitem,
 				      NULL);
     gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);
 
-//    fpm_directory = g_strjoin(NULL, g_get_home_dir(), "/.fpm2/", NULL);
     fpm_directory = g_build_filename (g_get_home_dir(), FPM_DIR, NULL);
 
     gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), fpm_directory);
@@ -826,10 +800,12 @@ on_preferences2_activate               (GtkMenuItem     *menuitem,
     }
 
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(lookup_widget (glb_win_misc, "startup_category")), ini->startup_category_active);
-	fpm_populate_combo_box(lookup_widget (glb_win_misc, "startup_category_combo"), ini->startup_category);
+	fpm_populate_combo_box(GTK_COMBO_BOX(lookup_widget (glb_win_misc, "startup_category_combo")), ini->startup_category);
 
-    fpm_copy_target_combo(lookup_widget(glb_win_misc, "copy_target_combo"), ini->copy_target);
+    fpm_copy_target_combo(GTK_COMBO_BOX(lookup_widget(glb_win_misc, "copy_target_combo")), ini->copy_target);
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(lookup_widget (glb_win_misc, "clear_target")), ini->clear_target);
+    gtk_combo_box_set_active (GTK_COMBO_BOX(lookup_widget(glb_win_misc, "dblclick_action_combo")), ini->dblclick_action);
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(lookup_widget (glb_win_misc, "startup_search")), ini->startup_search);
 
     if (gtk_dialog_run (GTK_DIALOG (glb_win_misc)) == GTK_RESPONSE_OK)
     {
@@ -899,15 +875,13 @@ on_dialog_password_delete_event        (GtkWidget       *widget,
                                         GdkEvent        *event,
                                         gpointer         user_data)
 {
-/*  gtk_main_quit();
-  return(TRUE);
-*/
   if(state->startup_tray) {
     gtk_widget_hide(gui->pass_window);
     state->locked = TRUE;
   } else
     gtk_main_quit();
 
+  return TRUE;
 }
 
 void
@@ -942,6 +916,7 @@ on_find_close_clicked                  (GtkToolButton   *toolbutton,
                                         gpointer         user_data)
 {
     gtk_widget_hide (GTK_WIDGET(lookup_widget (gui->main_window, "findbar")));
+    fpm_sensitive_menu(FALSE);
     fpm_clist_create_view(state->category);
 }
 
@@ -953,6 +928,7 @@ on_find_entry_key_press_event          (GtkWidget       *widget,
     // Escape
     if (event->keyval == 0xff1b) {
 	gtk_widget_hide (GTK_WIDGET(lookup_widget (gui->main_window, "findbar")));
+	fpm_sensitive_menu(FALSE);
 	fpm_clist_create_view(state->category);
     	return TRUE;
     }
@@ -966,6 +942,7 @@ on_find_entry_changed                  (GtkEditable     *editable,
 {
 	gchar *search_text;
 
+	fpm_sensitive_menu(FALSE);
 	search_text = gtk_editable_get_chars(editable, 0, -1);
 	fpm_search(search_text, FALSE);
 	g_free(search_text);
@@ -992,9 +969,12 @@ void
 tray_icon_on_click			(GtkStatusIcon *status_icon, 
 					 gpointer user_data)
 {
-    if (state->locked)
-	fpm_unlock();
-    else {
+    if (state->locked) {
+	if (GTK_WIDGET_VISIBLE(gui->pass_window))
+	    gtk_widget_hide(gui->pass_window);
+	else
+	    fpm_unlock();
+    } else {
 	if(state->minimized && ini->after_unhide_set_startup_category)
 	    fpm_switch_view_category(ini->startup_category);
     fpm_tr_toggle_main_window(FALSE);
@@ -1200,7 +1180,6 @@ on_select_key_file_clicked             (GtkButton       *button,
                                         gpointer         user_data)
 {
     GtkWidget *dialog;
-    GtkFileFilter *filter;
     const gchar *fpm_directory;
 
     dialog = gtk_file_chooser_dialog_new (_("Select Key File"),
@@ -1220,9 +1199,10 @@ on_select_key_file_clicked             (GtkButton       *button,
         filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
 	GtkComboBox *combo_box;
 
-        combo_box = GTK_COMBO_BOX(lookup_widget(glb_win_misc, "key_file_combo"));
+        combo_box = GTK_COMBO_BOX(lookup_widget(gtk_widget_get_toplevel(GTK_WIDGET(button)), "key_file_combo"));
 	gtk_combo_box_insert_text(combo_box, 1, filename);
         gtk_combo_box_set_active(combo_box, 1);
+
 	g_free (filename);
     }
     gtk_widget_destroy (dialog);
@@ -1280,6 +1260,7 @@ on_new_file_activate                   (GtkMenuItem     *menuitem,
     {
         glb_filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
 
+    gtk_widget_set_sensitive(gui->main_window, FALSE);
     fpm_launcher_remove_all();
     fpm_clear_list();
     fpm_clist_populate_cat_list();
@@ -1373,7 +1354,7 @@ on_btn_launcher_apply_clicked          (GtkButton       *button,
     fpm_launcher_populate_combo_box(g_list_index(glb_launcher_list, current_launcher));
     gtk_widget_set_sensitive(lookup_widget(glb_win_misc, "btn_launcher_add"), TRUE);
 
-    state->dirty = TRUE;
+    fpm_dirty(TRUE);
 
   } else {
     fpm_message(GTK_WINDOW(glb_win_misc), _("Please enter launcher name."), GTK_MESSAGE_WARNING);
@@ -1476,4 +1457,86 @@ on_clist_main_row_activated            (GtkTreeView     *treeview,
   fpm_double_click(glb_edit_data);
 }
 
+void
+on_clist_main_columns_changed          (GtkTreeView     *treeview,
+                                        gpointer         user_data)
+{
+
+  GtkTreeViewColumn *treeViewColumn = NULL;
+  GList *columns;
+  gint id;
+
+  g_list_free (columns_order);
+  columns_order = NULL;
+
+  columns = gtk_tree_view_get_columns(gui->main_clist);
+  while (columns != NULL) {
+	id = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(columns->data), "index"));
+	columns_order = g_list_append(columns_order, GINT_TO_POINTER(id));
+
+	treeViewColumn = GTK_TREE_VIEW_COLUMN(columns->data);
+
+	gtk_tree_view_column_set_resizable (treeViewColumn, TRUE);
+	gtk_tree_view_column_set_sizing (treeViewColumn, GTK_TREE_VIEW_COLUMN_FIXED);
+
+	columns = g_list_next(columns);
+   }
+
+  if(treeViewColumn != NULL)
+    gtk_tree_view_column_set_sizing (treeViewColumn, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+
+  g_list_free(columns);
+
+}
+
+
+void
+on_view_menu_activate                  (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+
+  gint id, i = 0;
+
+  id = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(menuitem), "index"));
+
+  if ((id == FPM_PASSWORD) && GTK_CHECK_MENU_ITEM(menuitem)->active ) {
+    if (!(fpm_question(GTK_WINDOW(gui->main_window), _("Are you sure you want decrypt and display ALL passwords?"))
+	== GTK_RESPONSE_YES)) {
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem), FALSE);
+	return;
+    }
+  }
+
+  fpm_view_modify_column(id, GTK_CHECK_MENU_ITEM (menuitem)->active);
+
+  GList *columns = gtk_tree_view_get_columns(gui->main_clist);
+  while (columns != NULL) {
+    ini->columns_width[i] = gtk_tree_view_column_get_width(columns->data);
+    columns = g_list_next(columns);
+    i++;
+  }
+  g_list_free(columns);
+  ini->columns_width[i-1] = 150;
+
+  fpm_clist_init();
+  fpm_clist_create_view(state->category);
+
+}
+
+
+void
+on_dblclick_action_combo_changed       (GtkComboBox     *combobox,
+                                        gpointer         user_data)
+{
+    ini->dblclick_action = gtk_combo_box_get_active(combobox);
+}
+
+
+void
+on_startup_search_toggled              (GtkToggleButton *togglebutton,
+                                        gpointer         user_data)
+{
+    ini->startup_search = togglebutton->active;
+
+}
 
