@@ -1,6 +1,6 @@
 /* FIGARO'S PASSWORD MANAGER 2 (FPM2)
  * Copyright (C) 2000 John Conneely
- * Copyright (C) 2008,2009 Aleš Koval
+ * Copyright (C) 2008-2010 Aleš Koval
  *
  * FPM is open source / free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,12 +28,10 @@
 #include "fpm_crypt.h"
 #include <fcntl.h>
 #include <sys/file.h>
-#include <sys/stat.h>
-#include <errno.h>
-#include <string.h>
 #include <stdlib.h>
 #include <unistd.h>
 
+#include <glib/gstdio.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 #include <X11/Xatom.h>
@@ -81,6 +79,20 @@ fpm_cipher_algo cipher_algo = AES256;
  */
 gboolean glb_need_decrypt;
 
+GList *columns_order = NULL;
+
+gchar *columns_title [] =
+{
+   "",
+   N_("Title") ,
+   N_("URL") ,
+   N_("User"),
+   N_("Password"),
+   N_("Category"),
+   N_("Notes") ,
+   N_("Launcher")
+};
+
 void
 lock_fpm_file(gchar* glb_filename)
 {
@@ -226,7 +238,8 @@ fpm_save_passitem(GtkWidget* win_edit, fpm_data* data)
   gboolean valid;
 
   /* Set dirty flag so we know we need to save later. */
-  state->dirty = TRUE;
+//  state->dirty = TRUE;
+  fpm_dirty(TRUE);
 
   /* First update data structures */
   data->title = fpm_get_entry(win_edit, "entry_title");
@@ -269,12 +282,12 @@ fpm_save_passitem(GtkWidget* win_edit, fpm_data* data)
   buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (notes));
   gtk_text_buffer_get_bounds (buffer, &start, &end);
   data->notes = gtk_text_iter_get_text (&start, &end);
-     
+
   model = gtk_tree_view_get_model(gui->main_clist);
   valid = gtk_tree_model_get_iter_first(model, &iter);
 
   while (valid) {
-    gtk_tree_model_get(model, &iter, FPM_COL_DATA_POINTER, &data1, -1);
+    gtk_tree_model_get(model, &iter, FPM_DATA_POINTER, &data1, -1);
     if( data == data1 ) {
 	row = atoi(gtk_tree_model_get_string_from_iter(model, &iter));
     }
@@ -298,7 +311,7 @@ fpm_save_passitem(GtkWidget* win_edit, fpm_data* data)
 	passfile_save(glb_filename);
   }
   else
-  { 
+  {
 //    g_print("updating...\n");
     if (ini->save_on_change)
 	passfile_save(glb_filename);
@@ -333,7 +346,6 @@ fpm_get_entry(GtkWidget *win, gchar *entry_name)
   return (tmp);
 }
 
-
 void
 fpm_set_entry(GtkWidget* win, gchar* entry_name, char* text)
 {
@@ -367,10 +379,26 @@ fpm_init(char* opt_file_name, int tray_on_startup)
 	fpm_toolbar_style();
 	gtk_widget_hide(GTK_WIDGET(lookup_widget (gui->main_window ,"hide_to_tray")));
 
+
 	gui->main_clist = GTK_TREE_VIEW(lookup_widget (gui->main_window, "clist_main"));
-	//  if(gtk_tree_view_get_model(gui->main_clist) == NULL)
+
 	fpm_clist_init();
 
+	gint i;
+        GtkWidget *view = gtk_menu_item_get_submenu(GTK_MENU_ITEM(lookup_widget (gui->main_window ,"view1")));
+        GList *children = gtk_container_get_children(GTK_CONTAINER(view));
+	for (i = 1; i < FPM_NUM_COLS; i++) {
+	    g_object_set_data(G_OBJECT(children->data), "index", GINT_TO_POINTER(i));
+
+	    if (g_list_find(columns_order, GINT_TO_POINTER(i)) != NULL) {
+		g_signal_handlers_block_by_func(G_OBJECT(children->data), G_CALLBACK(on_view_menu_activate), NULL);
+		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(children->data), TRUE);
+		g_signal_handlers_unblock_by_func(G_OBJECT(children->data), G_CALLBACK(on_view_menu_activate), NULL);
+	    }
+
+	    children = g_list_next(children);
+	}
+
 	state->new_entry = FALSE;
 	state->startup_tray = tray_on_startup;
 
@@ -435,6 +463,7 @@ void fpm_unlock() {
 
     label = GTK_LABEL(lookup_widget(gui->pass_window, "label_password_message"));
     gtk_widget_hide(GTK_WIDGET(label));
+    gtk_widget_grab_focus(lookup_widget(gui->pass_window, "entry_password"));
     gtk_widget_show(gui->pass_window);
 }
 
@@ -503,7 +532,7 @@ fpm_clipboard_clear(GtkClipboard *clip, gpointer data) {
 void
 fpm_clipboard_set(fpm_clipboard *clipboard) {
 
-    static const GtkTargetEntry targets[] = { "STRING", 0, 1 };
+    static const GtkTargetEntry targets[] = {{ "STRING", 0, 1 }};
     gchar *selection = _("primary selection");
     gchar *selection_type = _("user");
     gchar *dia_str;
@@ -549,8 +578,24 @@ fpm_clipboard_init(fpm_data *data, gint selection, gboolean is_password, gboolea
 void
 fpm_double_click(fpm_data* data)
 {
-  if (data != NULL)
-    fpm_jump(data);
+ switch(ini->dblclick_action) {
+    case ACTION_RUN_LAUNCHER:
+	if (data != NULL)
+	    fpm_jump(data);
+	break;
+    case ACTION_COPY_USERNAME:
+	fpm_clipboard_init(data, ini->copy_target, FALSE, FALSE);
+	break;
+    case ACTION_COPY_PASSWORD:
+	fpm_clipboard_init(data, ini->copy_target, TRUE, FALSE);
+	break;
+    case ACTION_COPY_BOTH:
+	fpm_clipboard_init(data, ini->copy_target, FALSE, TRUE);
+	break;
+    case ACTION_EDIT_ENTRY:
+	fpm_edit_passitem(&gui->edit_window, data);
+	break;
+ }
 }
 
 void
@@ -637,9 +682,9 @@ fpm_check_password()
   combo_box = GTK_COMBO_BOX(lookup_widget(gui->pass_window, "key_file_combo"));
 
   if(gtk_combo_box_get_active(combo_box)) {
-    gchar kf_hash[SHA256_DIGEST_LENGTH + 1] = {0};
+    guchar kf_hash[SHA256_DIGEST_LENGTH + 1] = {0};
 
-    if(sha256_file(gtk_combo_box_get_active_text(combo_box), kf_hash)) {
+    if(fpm_sha256_file(gtk_combo_box_get_active_text(combo_box), kf_hash)) {
 //	fpm_message(GTK_WINDOW(glb_win_misc), _("Cannot read key file!"), GTK_MESSAGE_ERROR);
 	fpm_message(GTK_WINDOW(gui->pass_window), _("Cannot read key file!"), GTK_MESSAGE_ERROR);
 	return;
@@ -663,7 +708,8 @@ fpm_check_password()
     glb_need_decrypt = FALSE;
     gchar* vhash;
     vhash = g_malloc0(cipher->hash_len + 1);
-    sha256_fpm_data(vhash);
+    fpm_sha256_fpm_data(vhash);
+
     fpm_decrypt_field(old_context, vstring_plain1, vstring, cipher->hash_len);
     if(strcmp(vstring_plain1, vhash) == 0) {
 	strcpy(vstring_plain,"FIGARO");
@@ -703,7 +749,7 @@ fpm_check_password()
 
 //	gtk_widget_hide(glb_win_misc);
 	gtk_widget_destroy(glb_win_misc);
-	
+
 	passfile_load(glb_filename);
 	fpm_crypt_init(password);
 	fpm_decrypt_launchers();
@@ -723,10 +769,10 @@ fpm_check_password()
   fpm_clist_populate_cat_list();
 
   gtk_window_resize (GTK_WINDOW(gui->main_window), ini->main_width, ini->main_height);
-  if ((ini->main_x != -1) && (ini->main_y != -1))
+  if((ini->main_x != -1) && (ini->main_y != -1) && !ini->dont_remember_position)
     gtk_window_move(GTK_WINDOW(gui->main_window), ini->main_x, ini->main_y);
 
-  if (ini->enable_tray_icon)
+  if(ini->enable_tray_icon)
     fpm_tray_icon();
 
   gtk_window_set_skip_taskbar_hint(GTK_WINDOW(gui->main_window), FALSE);
@@ -739,6 +785,12 @@ fpm_check_password()
 
   state->locked = FALSE;
   state->minimized = FALSE;
+
+  while (gtk_events_pending()) gtk_main_iteration();
+
+  if(ini->startup_search)
+    on_find_activate(NULL, NULL);
+
 }
 
 
@@ -768,9 +820,9 @@ void fpm_set_password()
 
   combo_box = GTK_COMBO_BOX(lookup_widget(glb_win_misc, "key_file_combo"));
   if(gtk_combo_box_get_active(combo_box)) {
-    gchar kf_hash[SHA256_DIGEST_LENGTH + 1] = {0}, *prehash;
+    guchar kf_hash[SHA256_DIGEST_LENGTH + 1] = {0};
 
-    if(sha256_file(gtk_combo_box_get_active_text(combo_box), kf_hash)) {
+    if(fpm_sha256_file(gtk_combo_box_get_active_text(combo_box), kf_hash)) {
 	fpm_message(GTK_WINDOW(glb_win_misc), _("Cannot read key file!"), GTK_MESSAGE_ERROR);
 	return;
     }
@@ -784,9 +836,10 @@ void fpm_set_password()
     state->category=g_strdup(FPM_ALL_CAT_MSG);
 
     fpm_crypt_init(pw1);
-    state->dirty = TRUE;
-//    gtk_widget_destroy(glb_win_misc);
+//    state->dirty = TRUE;
+    fpm_dirty(TRUE);
     gtk_widget_show(gui->main_window);
+    gtk_widget_set_sensitive(gui->main_window, TRUE);
 
     fpm_clist_populate_cat_list();
   }
@@ -932,8 +985,7 @@ void fpm_ini_load()
     GKeyFile *keyfile;
     GError *error = NULL;
     gsize count;
-    gint i;
-    gint *columns;
+    gint i, *columns;
 
     ini = g_malloc0(sizeof(fpm_ini));
 
@@ -977,6 +1029,10 @@ void fpm_ini_load()
 	ini->copy_target = SELECTION_PRIMARY;
 	ini->clear_target = TRUE;
 
+	ini->dont_remember_position = FALSE;
+	ini->dblclick_action = ACTION_RUN_LAUNCHER;
+	ini->startup_search = FALSE;
+
 	fpm_ini_save();
     } else {
 	ini->save_on_add = g_key_file_get_boolean (keyfile, "general", "save_on_add", NULL);
@@ -1011,14 +1067,28 @@ void fpm_ini_load()
 
 	error = NULL;
 	columns = g_key_file_get_integer_list (keyfile, "general", "columns_width", &count, &error);
-	if (error != NULL || count != FPM_NUM_COL || columns == NULL) {
-	    for (i=0; i< FPM_NUM_COL; i++) 
+	if (error != NULL || count != FPM_NUM_COLS || columns == NULL) {
+	    for (i = 0; i < FPM_NUM_COLS; i++)
 		ini->columns_width[i] = 150;
 	} else {
 	    memcpy(ini->columns_width, columns, sizeof(ini->columns_width));
 	    g_free(columns);
 	}
 
+	error = NULL;
+	columns = g_key_file_get_integer_list (keyfile, "general", "columns_order", &count, &error);
+	if (error != NULL || columns == NULL) {
+	    columns_order = g_list_append (columns_order, GINT_TO_POINTER (FPM_TITLE));
+	    columns_order = g_list_append (columns_order, GINT_TO_POINTER (FPM_URL));
+	    columns_order = g_list_append (columns_order, GINT_TO_POINTER (FPM_USER));
+	} else {
+	    for (i = 0; i < count; i++)
+		columns_order = g_list_append (columns_order, GINT_TO_POINTER (columns[i]));
+	    g_free(columns);
+	}
+
+        ini->dont_remember_position = g_key_file_get_boolean (keyfile, "general", "dont_remember_position", NULL);
+
 	error = NULL;
 	ini->search_in_title = g_key_file_get_boolean (keyfile, "search", "search_in_title", &error);
 	if (error != NULL)
@@ -1089,6 +1159,13 @@ void fpm_ini_load()
 	if (error != NULL)
 		ini->after_unhide_set_startup_category = FALSE;
 
+	error = NULL;
+	ini->dblclick_action = g_key_file_get_integer(keyfile, "general", "dblclick_action", &error);
+	if (error != NULL)
+		ini->dblclick_action = ACTION_RUN_LAUNCHER;
+
+        ini->startup_search = g_key_file_get_boolean (keyfile, "general", "startup_search", NULL);
+
 	g_free(fpm_ini_file);
 
     }
@@ -1099,25 +1176,43 @@ void fpm_ini_load()
 void fpm_ini_save()
 {
     GKeyFile *keyfile;
-    GtkTreeViewColumn *column;
     gint x, y, width, height;
-    gint i;
+    gint i = 0;
+    GList *columns;
+    gint columns_order_tmp[FPM_NUM_COLS];
 
     if (gui->main_window == NULL) {
 	x = -1;
 	y = -1;
 	width = 500;
 	height = 350;
-	for (i=0; i< FPM_NUM_COL; i++) 
+	for (i = 0; i < FPM_NUM_COLS; i++)
 	    ini->columns_width[i] = 150;
+
+        columns_order = g_list_append (columns_order, GINT_TO_POINTER (FPM_TITLE));
+        columns_order = g_list_append (columns_order, GINT_TO_POINTER (FPM_URL));
+        columns_order = g_list_append (columns_order, GINT_TO_POINTER (FPM_USER));
+
     } else {
 	gtk_window_get_size (GTK_WINDOW(gui->main_window), &width, &height);
 	gtk_window_get_position(GTK_WINDOW(gui->main_window), &x, &y);
 
-	for (i=0; i< FPM_NUM_COL; i++) {
-	    column = gtk_tree_view_get_column (gui->main_clist, i);
-	    ini->columns_width[i] = gtk_tree_view_column_get_width (column);
+	columns = gtk_tree_view_get_columns(gui->main_clist);
+	while (columns != NULL) {
+	    ini->columns_width[i] = gtk_tree_view_column_get_width (columns->data);
+	    columns = g_list_next(columns);
+	    i++;
 	}
+	g_list_free(columns);
+
+	i = 0;
+	columns = g_list_first(columns_order);
+	while(columns != NULL) {
+	    columns_order_tmp[i] = GPOINTER_TO_INT(columns->data);
+	    columns = g_list_next(columns);
+	    i++;
+	}
+
     }
 
     gchar *fpm_ini_file = g_build_filename (g_get_home_dir(), FPM_DIR, "fpm.ini", NULL);
@@ -1135,7 +1230,10 @@ void fpm_ini_save()
     g_key_file_set_integer (keyfile, "general", "main_y", y);
     g_key_file_set_integer (keyfile, "general", "main_width", width);
     g_key_file_set_integer (keyfile, "general", "main_height", height);
-    g_key_file_set_integer_list (keyfile, "general", "columns_width", ini->columns_width, 3);
+    g_key_file_set_integer_list (keyfile, "general", "columns_width", ini->columns_width, FPM_NUM_COLS);
+    g_key_file_set_integer_list (keyfile, "general", "columns_order", columns_order_tmp, i);
+    g_key_file_set_boolean (keyfile, "general", "dont_remember_position", ini->dont_remember_position);
+    g_key_file_set_integer (keyfile, "general", "dblclick_action", ini->dblclick_action);
 
     g_key_file_set_boolean (keyfile, "search", "search_in_title", ini->search_in_title);
     g_key_file_set_boolean (keyfile, "search", "search_in_url", ini->search_in_url);
@@ -1161,12 +1259,14 @@ void fpm_ini_save()
     g_key_file_set_integer (keyfile, "general", "copy_target", ini->copy_target);
     g_key_file_set_boolean (keyfile, "general", "clear_target", ini->clear_target);
 
+    g_key_file_set_boolean (keyfile, "general", "startup_search", ini->startup_search);
+
     gchar *save_data = g_key_file_to_data (keyfile, NULL, NULL);
 
     g_file_set_contents (fpm_ini_file, save_data, -1, NULL);
 
-	g_free(fpm_ini_file);    
-	g_free(save_data);
+    g_free(fpm_ini_file);
+    g_free(save_data);
     g_key_file_free (keyfile);
 }
 
@@ -1195,7 +1295,10 @@ void fpm_search(gchar *search_text, gboolean select_first)
 
         if ((!strcmp(state->category, data->category) && ini->search_limit_category)
 	    || ((!strcmp(state->category, FPM_ALL_CAT_MSG)
-	    || !ini->search_limit_category)))
+	    || !ini->search_limit_category)) 
+	    || (!strcmp(state->category, FPM_NONE_CAT_MSG) && !strlen(data->category))
+	    || (!strcmp(state->category, FPM_DEFAULT_CAT_MSG) && data->default_list)
+    	    )
 	{
 
 	    if (!ini->search_match_case) {
@@ -1213,12 +1316,7 @@ void fpm_search(gchar *search_text, gboolean select_first)
 
         	gtk_list_store_append (GTK_LIST_STORE(list_store), &iter);
 
-		gtk_list_store_set(GTK_LIST_STORE(list_store), &iter,
-	    	    FPM_COL_TITLE, data->title,
-    		    FPM_COL_URL, data->arg,
-	    	    FPM_COL_USER, data->user,
-	    	    FPM_COL_DATA_POINTER, data,
-	    	    -1);
+		fpm_clist_set_data (data, list_store, iter);
 	    }
 	}
 /*	if (!ini->search_match_case) {
@@ -1489,7 +1587,6 @@ gboolean fpm_is_launcher_in_category(fpm_launcher *launcher, gchar *category) {
 void fpm_switch_view_category(gchar *category) {
     GList* list;
     gint i = 0;
-    gint index = 0;
     gchar *dia_str;
 
     list = g_list_first(glb_cat_string_list);
@@ -1505,4 +1602,31 @@ void fpm_switch_view_category(gchar *category) {
     dia_str = g_strdup_printf(_("Passwords in category: %d"), glb_num_row);
     fpm_statusbar_push(dia_str);
     g_free(dia_str);
-}
\ No newline at end of file
+}
+
+void fpm_view_modify_column(gint type, gboolean active) {
+
+    if(active)
+	columns_order = g_list_append (columns_order, GINT_TO_POINTER(type));
+    else
+	columns_order = g_list_remove (columns_order, GINT_TO_POINTER(type));
+}
+
+void fpm_dirty(gboolean active) {
+    state->dirty = active;
+    gtk_widget_set_sensitive (lookup_widget(gui->main_window, "button_save"), active);
+    gtk_widget_set_sensitive (lookup_widget(gui->main_window, "save1"), active);
+}
+
+void fpm_sensitive_menu(gboolean active) {
+    gtk_widget_set_sensitive (lookup_widget(gui->main_window, "button_jump"), active);
+    gtk_widget_set_sensitive (lookup_widget(gui->main_window, "button_edit"), active);
+    gtk_widget_set_sensitive (lookup_widget(gui->main_window, "button_user"), active);
+    gtk_widget_set_sensitive (lookup_widget(gui->main_window, "button_pass"), active);
+    gtk_widget_set_sensitive (lookup_widget(gui->main_window, "copy_pass"), active);
+    gtk_widget_set_sensitive (lookup_widget(gui->main_window, "copy_user"), active);
+    gtk_widget_set_sensitive (lookup_widget(gui->main_window, "copy_both1"), active);
+    gtk_widget_set_sensitive (lookup_widget(gui->main_window, "edit_item1"), active);
+    gtk_widget_set_sensitive (lookup_widget(gui->main_window, "clone_item1"), active);
+    gtk_widget_set_sensitive (lookup_widget(gui->main_window, "item_delete"), active);
+}
