@@ -1,6 +1,6 @@
 /* FIGARO'S PASSWORD MANAGER 2 (FPM2)
  * Copyright (C) 2000 John Conneely
- * Copyright (C) 2009 Aleš Koval
+ * Copyright (C) 2009,2010 Aleš Koval
  *
  * FPM is open source / free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -132,8 +132,8 @@ fpm_file_save(gchar* file_name, gboolean convert)
   GList * list;
   gint i;
   gchar *dia_str;
-  gchar* vstring_cipher;
-  gchar* vstring_plain;
+  gchar *vstring_cipher = "";
+  gchar *vstring_plain = "";
 
   if (ini->create_backup) 
     move_backup(file_name);
@@ -161,7 +161,7 @@ fpm_file_save(gchar* file_name, gboolean convert)
   } else if(cipher_algo == AES256) {
     vstring_cipher = g_malloc0(cipher->hash_len * 2 + 1);
     vstring_plain = g_malloc0(cipher->hash_len + 1);
-    sha256_fpm_data(vstring_plain);
+    fpm_sha256_fpm_data(vstring_plain);
     fpm_encrypt_field(new_context, vstring_cipher, vstring_plain, cipher->hash_len);
   }
     xmlSetProp(item, (xmlChar *)"vstring", (xmlChar *)vstring_cipher);
@@ -174,7 +174,7 @@ fpm_file_save(gchar* file_name, gboolean convert)
   while (list!=NULL)
   {
     launcher=list->data;
-    item=xmlNewChild(nodelist, NULL, (xmlChar *)"LauncerItem", NULL);
+    item=xmlNewChild(nodelist, NULL, (xmlChar *)"LauncherItem", NULL);
     if(cipher_algo == BLOWFISH) {
 	new_leaf(doc, item, (xmlChar *)"title", (xmlChar *)launcher->title);
 	new_leaf(doc, item, (xmlChar *)"cmdline", (xmlChar *)launcher->cmdline);
@@ -184,13 +184,13 @@ fpm_file_save(gchar* file_name, gboolean convert)
 	tmp=g_strdup_printf("%d", launcher->copy_password);
 	new_leaf(doc, item, (xmlChar *)"copy_password", (xmlChar *)tmp);
     } else {
-	new_leaf_encrypt(doc, item, (xmlChar *)"title", (xmlChar *)launcher->title);
-	new_leaf_encrypt(doc, item, (xmlChar *)"cmdline", (xmlChar *)launcher->cmdline);
+	new_leaf_encrypt(doc, item, (xmlChar *)"title", launcher->title);
+	new_leaf_encrypt(doc, item, (xmlChar *)"cmdline", launcher->cmdline);
 	tmp=g_strdup_printf("%d", launcher->copy_user);
-	new_leaf_encrypt(doc, item, (xmlChar *)"copy_user", (xmlChar *)tmp);
+	new_leaf_encrypt(doc, item, (xmlChar *)"copy_user", tmp);
 	g_free(tmp);
 	tmp=g_strdup_printf("%d", launcher->copy_password);
-	new_leaf_encrypt(doc, item, (xmlChar *)"copy_password", (xmlChar *)tmp);
+	new_leaf_encrypt(doc, item, (xmlChar *)"copy_password", tmp);
     }
     g_free(tmp);
     list = g_list_next(list);
@@ -233,9 +233,10 @@ fpm_file_save(gchar* file_name, gboolean convert)
   fpm_statusbar_push(dia_str);
 
 //  printf("Saved %d password(s).\n", i);
-  state->dirty = FALSE;
+//  state->dirty = FALSE;
   xmlFreeDoc(doc);
   xmlCleanupParser();
+  fpm_dirty(FALSE);
 }
 
 void
@@ -275,7 +276,7 @@ fpm_file_export(gchar *file_name, gint export_launchers, gchar *export_category)
   {
     launcher = list->data;
     if((export_launchers == 2) || ((export_launchers == 1) && fpm_is_launcher_in_category(launcher, export_category))) {
-    item = xmlNewChild(nodelist, NULL, (xmlChar *)"LauncerItem", NULL);
+    item = xmlNewChild(nodelist, NULL, (xmlChar *)"LauncherItem", NULL);
     new_leaf(doc, item, (xmlChar *)"title", (xmlChar *)launcher->title);
     new_leaf(doc, item, (xmlChar *)"cmdline", (xmlChar *)launcher->cmdline);
     tmp = g_strdup_printf("%d", launcher->copy_user);
@@ -356,7 +357,8 @@ passfile_load(gchar* file_name)
     old_salt=get_new_salt(cipher->salt_len);
     new_salt=old_salt;
     glb_pass_list=NULL;
-    state->dirty=TRUE;
+//    state->dirty=TRUE;
+    fpm_dirty(TRUE);
     fpm_init_launchers();
     return(-1);
   }
@@ -536,10 +538,10 @@ passfile_load(gchar* file_name)
 //  fpm_statusbar_push(dia_str);
 //  g_free(dia_str);
 
-  state->dirty = FALSE;
+//  state->dirty = FALSE;
+  fpm_dirty(FALSE);
 
   return(0);
-
 }
 
 static void new_leaf_encrypt_password
@@ -709,10 +711,10 @@ fpm_file_import(gchar *file_name, gint import_launchers, gchar *import_category,
 
       if(!strcmp((char *)attr->name, "password")) {
 	gchar *xmlplain;
-	
-	xmlplain = xmlNodeListGetString(doc, attr->xmlChildrenNode, 1);
-	if( xmlplain != NULL) {
-	    strncpy(plaintext, xmlNodeListGetString(doc, attr->xmlChildrenNode, 1) , FPM_PASSWORD_LEN);
+
+	xmlplain = (gchar *)xmlNodeListGetString(doc, attr->xmlChildrenNode, 1);
+	if(xmlplain != NULL) {
+	    strncpy(plaintext, xmlplain , FPM_PASSWORD_LEN);
 	} else {
 	    strcpy(plaintext,"");
 	}
@@ -733,7 +735,8 @@ fpm_file_import(gchar *file_name, gint import_launchers, gchar *import_category,
   xmlCleanupParser();
 /*  free(file_version); */
 
-  state->dirty = TRUE;
+//  state->dirty = TRUE;
+  fpm_dirty(TRUE);
 
   *message = g_strdup_printf(_("Imported %d password(s)."), i);
 
